==> bob.learn.pytorch-0.0.3.zip: bob.learn.pytorch-0.0.3/bob/learn/pytorch/scripts/train_cnn.py <==
#!/usr/bin/env python
# encoding: utf-8


""" Train a CNN for face recognition

Usage:
  %(prog)s <configuration> 
            [--model=<string>] [--batch-size=<int>] [--epochs=<int>] 
            [--learning-rate=<float>] [--seed=<int>] 
            [--output-dir=<path>] [--use-gpu] [--verbose ...]

Arguments:
  <configuration>  A configuration file, defining the dataset and the network

Options:
  -h, --help                    Shows this help message and exits
      --model=<string>          Filename of the model to load (if any). 
      --batch-size=<int>        Batch size [default: 64]
      --epochs=<int>            Number of training epochs [default: 20]
      --learning-rate=<float>   Learning rate [default: 0.01]
  -S, --seed=<int>              The random seed [default: 3] 
  -o, --output-dir=<path>       Dir to save stuff [default: training]
  -g, --use-gpu                 Use the GPU
  -v, --verbose                 Increase the verbosity (may appear multiple times).

Note that arguments provided directly by command-line will override the ones in the configuration file.

Example:

  To run the training process 

    $ %(prog)s config.py 

See '%(prog)s --help' for more information.

"""

import os, sys
import pkg_resources

import torch
import numpy
from docopt import docopt

import bob.core
logger = bob.core.log.setup("bob.learn.pytorch")

from bob.extension.config import load
from bob.learn.pytorch.trainers import CNNTrainer

version = pkg_resources.require('bob.learn.pytorch')[0].version


def get_parameter(args, configuration, keyword, default):
  """ Get the right value for a parameter

  The parameters are either defined in a separate configuration file
  or diven directly via command-line. Note that the command-line
  has priority over the configuration file.

  As a convention, parameters made with more than one word (i.e. batch size)
  are provided with an underscore in the configuration file, and with an
  hyphen in the command-line:

    - configuration:  batch_size=64
    - command line:   --batch-size=64

  Parameters
  ----------
  args: dictionary
    The arguments as parsed from the command line.
  configuration: object
    The arguments given by the configuration file.
  keyword: string
    the keyword for the parameter to process (in the "configuration" style)
  default: 
    The default value of the parameter

  Returns
  -------
  arg:
    The right value for the given keyword argument

  """
 
  args_kw = '--' + keyword.replace('_', '-')
  _type = type(default)
  arg = _type(args[args_kw])
  if hasattr(configuration, keyword):
    arg = getattr(configuration, keyword)
  if _type(args[args_kw]) is not default:
    arg = _type(args[args_kw])
  return arg


def main(user_input=None):
  
  # Parse the command-line arguments
  if user_input is not None:
      arguments = user_input
  else:
      arguments = sys.argv[1:]

  prog = os.path.basename(sys.argv[0])
  completions = dict(prog=prog, version=version,)
  args = docopt(__doc__ % completions,argv=arguments,version='Train a CNN (%s)' % version,)

  # load configuration file
  configuration = load([os.path.join(args['<configuration>'])])
  
  # get the pre-trained model file, if any
  model = args['--model']
  if hasattr(configuration, 'model'):
    model = configuration.model
  
  # get various parameters, either from config file or command-line 
  batch_size = get_parameter(args, configuration, 'batch_size', 64)
  epochs = get_parameter(args, configuration, 'epochs', 20)
  learning_rate = get_parameter(args, configuration, 'learning_rate', 0.01)
  seed = get_parameter(args, configuration, 'seed', 3)
  output_dir = get_parameter(args, configuration, 'output_dir', 'training')
  use_gpu = get_parameter(args, configuration, 'use_gpu', False)
  verbosity_level = get_parameter(args, configuration, 'verbose', 0)
  
  bob.core.log.set_verbosity_level(logger, verbosity_level)
  bob.io.base.create_directories_safe(output_dir)

  # print parameters
  logger.debug("Model file = {}".format(model))
  logger.debug("Batch size = {}".format(batch_size))
  logger.debug("Epochs = {}".format(epochs))
  logger.debug("Learning rate = {}".format(learning_rate))
  logger.debug("Seed = {}".format(seed))
  logger.debug("Output directory = {}".format(output_dir))
  logger.debug("Use GPU = {}".format(use_gpu))

  # process on the arguments / options
  torch.manual_seed(seed)
  if use_gpu:
    torch.cuda.manual_seed_all(seed)
  if torch.cuda.is_available() and not use_gpu:
    logger.warn("You have a CUDA device, so you should probably run with --use-gpu")

  # get data
  if hasattr(configuration, 'dataset'):
    dataloader = torch.utils.data.DataLoader(configuration.dataset, batch_size=batch_size, shuffle=True)
    logger.info("There are {} training images from {} categories".format(len(configuration.dataset), numpy.max(configuration.dataset.id_labels)))
  else:
    logger.error("Please provide a dataset in your configuration file !")
    sys.exit()
  
  # train the network
  if hasattr(configuration, 'network'):
    trainer = CNNTrainer(configuration.network, batch_size=batch_size, use_gpu=use_gpu, verbosity_level=verbosity_level)
    trainer.train(dataloader, n_epochs=epochs, learning_rate=learning_rate, output_dir=output_dir, model=model)
  else:
    logger.error("Please provide a network in your configuration file !")
    sys.exit()

==> bob.learn.pytorch-0.1.2.zip: bob.learn.pytorch-0.1.2/bob/learn/pytorch/scripts/train_cnn.py <==
#!/usr/bin/env python
# encoding: utf-8


""" Train a CNN for face recognition

Usage:
  %(prog)s <configuration> 
            [--model=<string>] [--batch-size=<int>] [--epochs=<int>] 
            [--learning-rate=<float>] [--seed=<int>] 
            [--output-dir=<path>] [--use-gpu] [--verbose ...]

Arguments:
  <configuration>  A configuration file, defining the dataset and the network

Options:
  -h, --help                    Shows this help message and exits
      --model=<string>          Filename of the model to load (if any). 
      --batch-size=<int>        Batch size [default: 64]
      --epochs=<int>            Number of training epochs [default: 20]
      --learning-rate=<float>   Learning rate [default: 0.01]
  -S, --seed=<int>              The random seed [default: 3] 
  -o, --output-dir=<path>       Dir to save stuff [default: training]
  -g, --use-gpu                 Use the GPU
  -v, --verbose                 Increase the verbosity (may appear multiple times).

Note that arguments provided directly by command-line will override the ones in the configuration file.

Example:

  To run the training process 

    $ %(prog)s config.py 

See '%(prog)s --help' for more information.

"""

import os, sys
import pkg_resources

import torch
import numpy
from docopt import docopt

import bob.core
logger = bob.core.log.setup("bob.learn.pytorch")

from bob.extension.config import load
from bob.learn.pytorch.trainers import CNNTrainer
from bob.learn.pytorch.utils import get_parameter

version = pkg_resources.require('bob.learn.pytorch')[0].version

def main(user_input=None):
  
  # Parse the command-line arguments
  if user_input is not None:
      arguments = user_input
  else:
      arguments = sys.argv[1:]

  prog = os.path.basename(sys.argv[0])
  completions = dict(prog=prog, version=version,)
  args = docopt(__doc__ % completions,argv=arguments,version='Train a CNN (%s)' % version,)

  # load configuration file
  configuration = load([os.path.join(args['<configuration>'])])
  
  # get the pre-trained model file, if any
  model = args['--model']
  if hasattr(configuration, 'model'):
    model = configuration.model
  
  # get various parameters, either from config file or command-line 
  batch_size = get_parameter(args, configuration, 'batch_size', 64)
  epochs = get_parameter(args, configuration, 'epochs', 20)
  learning_rate = get_parameter(args, configuration, 'learning_rate', 0.01)
  seed = get_parameter(args, configuration, 'seed', 3)
  output_dir = get_parameter(args, configuration, 'output_dir', 'training')
  use_gpu = get_parameter(args, configuration, 'use_gpu', False)
  verbosity_level = get_parameter(args, configuration, 'verbose', 0)
  
  bob.core.log.set_verbosity_level(logger, verbosity_level)
  bob.io.base.create_directories_safe(output_dir)

  # print parameters
  logger.debug("Model file = {}".format(model))
  logger.debug("Batch size = {}".format(batch_size))
  logger.debug("Epochs = {}".format(epochs))
  logger.debug("Learning rate = {}".format(learning_rate))
  logger.debug("Seed = {}".format(seed))
  logger.debug("Output directory = {}".format(output_dir))
  logger.debug("Use GPU = {}".format(use_gpu))

  # process on the arguments / options
  torch.manual_seed(seed)
  if use_gpu:
    torch.cuda.manual_seed_all(seed)
  if torch.cuda.is_available() and not use_gpu:
    logger.warn("You have a CUDA device, so you should probably run with --use-gpu")

  # get data
  if hasattr(configuration, 'dataset'):
    dataloader = torch.utils.data.DataLoader(configuration.dataset, batch_size=batch_size, shuffle=True)
    num_classes = numpy.max(configuration.dataset.id_labels)
    logger.info("There are {} training images from {} categories".format(len(configuration.dataset), num_classes))
  else:
    logger.error("Please provide a dataset in your configuration file !")
    sys.exit()
  
  # train the network
  if hasattr(configuration, 'network'):
    trainer = CNNTrainer(configuration.network, batch_size=batch_size, use_gpu=use_gpu, verbosity_level=verbosity_level, num_classes=num_classes)
    trainer.train(dataloader, n_epochs=epochs, learning_rate=learning_rate, output_dir=output_dir, model=model)
  else:
    logger.error("Please provide a network in your configuration file !")
    sys.exit()
